%Farrow interpolation test.
%
%Rate converts blocks of inputs to outputs.  The input block size is fixed but the output
%block size varies depending on how many outputs can be produced from the inputs.

% K - polynomial order for filter coefficients
K = 5;

% L - filter order (L+1 tap)
L = 6;

% design the M oversampled prototype filter
% the oversampling is so a polynomial can be fit through the points
% the filter length should be L*M+1 for some integer M, the L*M order guarantees it
% the filter response itself depends on the application, this is just an example
M = 64;
h = remez(L*M,[0 .5/M 1.5/M 1],[1 1 0 0]);
h = h/max(h);

% create the piecewise polynomial approximation to the desired filter
[P e] = construct_farrow(h,L,K);

% the approximated filter is the prototype plus the error
hh = h+e;

% dt = output time step (relative to the input time step which is always assumed to be 1)
dt = .987;

% Nx = block size for processing
Nx = 123;

% initial conditions
x0 = zeros(1,L-1);
t0 = 0;
tx = [0];

% for some number of blocks...
for i=1:25,
  % get the input samples
  tx = tx(length(tx))+[1:Nx];
  x = sin(tx/211*7*2*pi);

  % rate convert the inputs
  if 1,
    [y t Ny t0 x0] = asrc_farrow(x,dt,t0,x0,P);
  else,
    [y t Ny t0 x0] = asrc_farrow_loop(x,dt,t0,x0,P);
  end;

  % fyi print output block size
  Ny

  % plot input and output data
  plot([1:Nx],x);
  hold on;
  plot(t,y,'.r');
  hold off;
  pause;
end;